"""
MIT License

Copyright (c) 2023 qvco

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

from .. import client
from ..responses import (
    BgmsResponse,
    CallStatusResponse,
    ConferenceCallResponse,
    GamesResponse,
    GenresResponse,
    PostResponse,
    PostsResponse,
    UsersByTimestampResponse,
)


class CallAPI(object):
    def __init__(self, base: client.BaseClient) -> None:
        self.__base = base

    def bump_call(self, call_id: int, participant_limit: int = None):
        params = {}
        if participant_limit:
            params["participant_limit"] = participant_limit
        return self.__base._request(
            "POST", route=f"/v1/calls/{call_id}/bump", params=params
        )

    def get_user_active_call(self, user_id: int) -> PostResponse:
        return self.__base._request(
            "GET",
            route=f"/v1/posts/active_call",
            params={"user_id": user_id},
            data_type=PostResponse,
        )

    def get_bgms(self) -> BgmsResponse:
        return self.__base._request(
            "GET", route="/v1/calls/bgm", data_type=BgmsResponse
        )

    def get_call(self, call_id: int) -> ConferenceCallResponse:
        return self.__base._request(
            "GET",
            route=f"/v1/calls/conferences/{call_id}",
            data_type=ConferenceCallResponse,
        )

    def get_call_invitable_users(
        self, call_id: int, from_timestamp: int = None
    ) -> UsersByTimestampResponse:
        # @Nullable @Query("user[nickname]")
        params = {}
        if from_timestamp:
            params["from_timestamp"] = from_timestamp
        return self.__base._request(
            "GET",
            route=f"/v1/calls/{call_id}/users/invitable",
            params=params,
            data_type=UsersByTimestampResponse,
        )

    def get_call_status(self, opponent_id: int) -> CallStatusResponse:
        return self.__base._request(
            "GET",
            route=f"/v1/calls/phone_status/{opponent_id}",
            data_type=CallStatusResponse,
        )

    def get_games(self, **params) -> GamesResponse:
        """

        Parameters
        ----------
            - number: int - (optional)
            - ids: list[int] - (optional)
            - from_id: int - (optional)

        """
        return self.__base._request(
            "GET",
            route=f"/v1/games/apps",
            params=params,
            data_type=GamesResponse,
        )

    def get_genres(self, **params) -> GenresResponse:
        """

        Parameters
        ----------
            - number: int - (optional)
            - from: int - (optional)

        """
        return self.__base._request(
            "GET",
            route=f"/v1/genres",
            params=params,
            data_type=GenresResponse,
        )

    def get_group_calls(self, **params) -> PostsResponse:
        """

        Parameters
        ----------
            - number: int - (optional)
            - group_category_id: int - (optional)
            - from_timestamp: int - (optional)
            - scope: str - (optional)

        """
        return self.__base._request(
            "GET",
            route="/v1/posts/group_calls",
            params=params,
            data_type=PostsResponse,
        )

    def invite_to_call_bulk(self, call_id: int, group_id: int = None):
        """

        Parameters
        ----------
            - call_id: int - (required)
            - group_id: int - (optional)

        """
        params = {}
        if group_id:
            params["group_id"] = group_id
        return self.__base._request(
            "POST",
            route=f"/v1/calls/{call_id}/bulk_invite",
            params=params,
        )

    def invite_users_to_call(self, call_id: int, user_ids: list[int]):
        """

        Parameters
        ----------
            - call_id: int - (required)
            - user_ids: list[int] - (required)

        """
        return self.__base._request(
            "POST",
            route=f"/v1/calls/conference_calls/{call_id}/invite",
            payload={"call_id": call_id, "user_ids": user_ids},
        )

    def invite_users_to_chat_call(self, chat_room_id: int, room_id: int, room_url: str):
        return self.__base._request(
            "POST",
            route="/v2/calls/invite",
            payload={
                "chat_room_id": chat_room_id,
                "room_id": room_id,
                "room_url": room_url,
            },
        )

    def kick_and_ban_from_call(self, call_id: int, user_id: int):
        return self.__base._request(
            "POST",
            route=f"/v1/calls/conference_calls/{call_id}/kick",
            payload={"user_id": user_id},
        )

    def set_call(
        self,
        call_id: int,
        joinable_by: str,
        game_title: str = None,
        category_id: str = None,
    ):
        return self.__base._request(
            "PUT",
            route=f"/v1/calls/{call_id}",
            payload={
                "joinable_by": joinable_by,
                "game_title": game_title,
                "category_id": category_id,
            },
        )

    def set_user_role(self, call_id: int, user_id: int, role: str):
        return self.__base._request(
            "PUT",
            route=f"/v1/calls/{call_id}/users/{user_id}",
            payload={"role": role},
        )

    def start_call(
        self, conference_id: int, call_sid: str = None
    ) -> ConferenceCallResponse:
        return self.__base._request(
            "POST",
            route="/v1/calls/start_conference_call",
            payload={"conference_id": conference_id, "call_sid": call_sid},
            data_type=ConferenceCallResponse,
        )

    def start_anonymous_call(
        self, conference_id: int, agora_uid: str
    ) -> ConferenceCallResponse:
        return self.__base._request(
            "POST",
            route="/v1/anonymous_calls/start_conference_call",
            payload={"conference_id": conference_id, "agora_uid": agora_uid},
            data_type=ConferenceCallResponse,
        )

    def stop_anonymous_call(self, conference_id: int, agora_uid: str = None):
        return self.__base._request(
            "POST",
            route="/v1/anonymous_calls/leave_conference_call",
            payload={"conference_id": conference_id, "agora_uid": agora_uid},
        )

    def stop_call(self, conference_id: int, call_sid: str = None):
        return self.__base._request(
            "POST",
            route="/v1/calls/leave_conference_call",
            payload={"conference_id": conference_id, "call_sid": call_sid},
        )



"""
MIT License

Copyright (c) 2023 qvco

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from enum import Enum


__all__ = (
    "YaylibError",
    "HTTPError",
    "BadRequestError",
    "AuthenticationError",
    "ForbiddenError",
    "NotFoundError",
    "RateLimitError",
    "YayServerError",
    "ErrorCode",
)


class YaylibError(Exception):
    """Base exception for yaylib"""

    pass


class HTTPError(YaylibError):
    """HTTPError()

    Exception raised when an HTTP request fails
    """

    pass


class BadRequestError(HTTPError):
    """BadRequestError()

    Exception raised for a 400 HTTP status code
    """

    pass


class AuthenticationError(HTTPError):
    """AuthenticationError()

    Exception raised for a 401 HTTP status code
    """

    pass


class ForbiddenError(HTTPError):
    """ForbiddenError()

    Exception raised for a 403 HTTP status code
    """

    pass


class NotFoundError(HTTPError):
    """NotFoundError()

    Exception raised for a 404 HTTP status code
    """

    pass


class RateLimitError(HTTPError):
    """RateLimitError()

    Exception raised for a 429 HTTP status code
    """

    pass


class YayServerError(HTTPError):
    """YayServerError()

    Exception raised for a 5xx HTTP status code
    """

    pass


class ErrorCode(Enum):
    Unknown = "unknown"
    InvalidParameter = -1
    RegisteredUser = -2
    AccessTokenExpired = -3
    ScreenNameAlreadyBeenTaken = -4
    UserNotFound = -5
    PostNotFound = -6
    ChatRoomNotFound = -7
    ChatMessageNotFound = -8
    UserNotFoundAtChatRoom = -9
    UserMustBeOverTwoAtChatRoom = -10
    IncorrectPassword = -11
    UserBlocked = -12
    PrivateUser = -13
    ApplicationNotFound = -14
    BadSNSCredentials = -15
    SNSAlreadyConnected = -16
    CannotDisconnectSNS = -17
    AccessTokenInvalid = -18
    SpotNotFound = -19
    UserBanned = -20
    UserTemporaryBanned = -21
    SchoolInfoChange = -22
    CannotDeleteNewUser = -26
    CaptchaRequired = -29
    FailedToVerifyCaptcha = -30
    GroupIsFull = -100
    BannedFromGroup = -103
    InvalidCurrentPassword = -200
    InvalidPassword = -201
    InvalidEmailOrPassword = -202
    ExistEmail = -203
    BadEmailReputation = -204
    ChatRoomIsFull = -308
    ConferenceIsFull = -309
    ConferenceInactive = -310
    GroupOwnerBlockedYou = -312
    ChatNeedMutualFollowed = -313
    ConferenceCallIsLocked = -315
    ConferenceCallIsForFollowersOnly = -317
    InvalidEmail = -319
    RegisteredEmail = -320
    BannedFromCall = -321
    NotCallOwner = -322
    NotVipUser = -326
    BlockingLimitExceeded = -331
    VerificationCodeWrong = -332
    VerificationCodeExpired = -333
    InvalidAppVersion = -334
    InvalidPhoneNumber = -335
    FollowLimitation = -336
    AgeGapNotAllowed = -338
    GroupOwnerOrGroupModeratorOnly = -339
    UnableToRegisterUserDueToPolicy = -340
    SnsShareRewardAlreadyBeenClaimed = -342
    QuotaLimitExceeded = -343
    ChatNeedAgeVerified = -346
    OnlyAgeVerifiedUserCanJoinGroup = -347
    RequirePhoneVerificationToChat = -348
    NotPostOwner = -350
    GroupGenerationNotMatched = -352
    PhoneNumberCheckVerificationCodeSubmitQuotaExceeded = -355
    PhoneNumberCheckVerificationCodeRequestQuotaExceeded = -356
    GroupOfferHasBeenAccepted = -357
    GroupOfferHasBeenWithdrawn = -358
    IpBanned = -360
    NotConnectedToTwitter = -361
    PrivateUserTimeline = -363
    CounterRefreshLimitExceeded = -364
    NotFollowedByOpponent = -367
    ExceedChangeCountryQuota = -369
    NotGroupMember = -370
    GroupPendingTransfer = -371
    GroupPendingDeputization = -372
    UserRestrictedChatWithCautionUsers = -373
    RestrictedCreateChatWithNewUsers = -374
    RepostPostNotRepostable = -375
    TooManyAccountsCreated = -376
    OnlySpecificGenderCanJoinGroup = -377
    CreateSpecificGenderGroupRequiredGender = -378
    GroupRelatedExceededNumberOfRelatedGroups = -382
    ExceededPinnedLimit = -383
    GroupShareOnTwitterLimitExceeded = -384
    ReportedContent = -385
    ConferenceCallIsForMutualFollowsOnly = -402
    ExceededLimit = -403
    GroupInviteExceeded = -404
    PhoneVerificationRequired = -405
    ContentTooOld = -406
    PasswordTooShort = -407
    PasswordTooLong = -408
    PasswordNotAllowed = -409
    CommonPassword = -410
    EmailNotAuthorized = -411
    UnableToMovePostToThread = -412
    UnableToPostUrl = -413
    UnableToSetCall = -977
    PhoneNumberBanned = -1000
    TooManyRequests = -5302


class ErrorMessage(Enum):
    Unknown = "原因不明"
    RegisteredUser = "このアカウントはすでに登録されています"
    AccessTokenExpired = "アクセストークンの有効期限切れ"
    ScreenNameAlreadyBeenTaken = "このIDはすでに使われています"
    UserNotFound = "ユーザーが見つかりません"
    PostNotFound = "投稿が見つかりません"
    ChatRoomNotFound = "チャットルームが見つかりません"
    ChatMessageNotFound = "メッセージが見つかりません"
    UserNotFoundAtChatRoom = "チャットルームに特定のユーザーが見つかりません"
    UserMustBeOverTwoAtChatRoom = "チャットルーム内のユーザーは2人以上の必要があります"
    IncorrectPassword = "正しいパスワードを確認してください"
    UserBlocked = "このユーザーはブロック中です"
    PrivateUser = "プライベートユーザーです"
    ApplicationNotFound = "アプリケーションが見つかりません"
    BadSNSCredentials = "アカウント認証に失敗しました"
    SNSAlreadyConnected = "このSNSアカウントはすでに使われています"
    CannotDisconnectSNS = "SNSの連携を解除できません"
    AccessTokenInvalid = "アクセストークンが不正です"
    SpotNotFound = "スポットが見つかりません"
    UserBanned = "このアカウントは永久的に凍結されました"
    UserTemporaryBanned = "サービスの規約に抵触したため、アカウントを停止しています"
    SchoolInfoChange = "学校の情報が変更されています"
    CannotDeleteNewUser = "アカウントを作成して3日以内に削除はできません"
    CaptchaRequired = "Captcha認証が必要です"
    FailedToVerifyCaptcha = "Captcha認証に失敗しました"
    GroupIsFull = "サークルが満員です"
    BannedFromGroup = "このサークルから追放されています"
    InvalidCurrentPassword = "現在のパスワードを入力してください"
    InvalidPassword = "現在のパスワードを確認してください"
    InvalidEmailOrPassword = "メールアドレス、もしくはパスワードが不正です"
    ExistEmail = "このメールアドレスはすでに使われています"
    BadEmailReputation = "スパムメールの可能性があり、操作を完了できません"
    ChatRoomIsFull = "グルチャの最大人数は50人です"
    ConferenceIsFull = "通話が満員です"
    ConferenceInactive = "通話は終了しました"
    GroupOwnerBlockedYou = "サークルの管理人にブロックされています"
    ChatNeedMutualFollowed = "チャットするにはフォローされる必要があります"
    ConferenceCallIsLocked = "通話はロックされています"
    ConferenceCallIsForFollowersOnly = "枠主をフォローすることで参加できます"
    InvalidEmail = "正しいメールアドレスを確認してください"
    RegisteredEmail = "このメールアドレスはすでに登録されています"
    BannedFromCall = "通話に参加できません。永久退出処分を受けています。"
    NotCallOwner = "通話のオーナーではありません"
    NotVipUser = "VIPユーザーではありません"
    BlockingLimitExceeded = "ブロック数の上限に達しました"
    VerificationCodeWrong = "認証コードが不正です"
    VerificationCodeExpired = "認証コードの有効期限切れ"
    InvalidAppVersion = "アプリをアップデートしてください"
    InvalidPhoneNumber = "この機能を使用するためには電話番号認証が必要です"
    FollowLimitation = "制限に達したので、これ以上フォローすることができません。この制限はフォローワーを増やすことで、増やすことができます。"
    AgeGapNotAllowed = "レター、もしくはチャットを送れません。年齢が離れすぎています"
    GroupOwnerOrGroupModeratorOnly = "サークル管理人か副管理人のみ可能な操作です"
    UnableToRegisterUserDueToPolicy = "利用規約に基づき、現在アカウントを登録できません"
    SnsShareRewardAlreadyBeenClaimed = "SNS共有の特典は既に取得済みです"
    QuotaLimitExceeded = "この機能の上限回数に達しました。1時間ほど時間を置いて再度お試しください。"
    ChatNeedAgeVerified = "チャット送信に年齢確認が必要です"
    OnlyAgeVerifiedUserCanJoinGroup = "このサークルに参加するには年齢確認をする必要があります"
    RequirePhoneVerificationToChat = "チャットをするには電話番号認証が必要です"
    NotPostOwner = "編集は投稿の作成者のみ可能です"
    GroupGenerationNotMatched = "特定の年齢層のみ参加が許可されているサークルです"
    PhoneNumberCheckVerificationCodeSubmitQuotaExceeded = (
        "認証コードの送信回数が上限を越えました。1時間ほど時間をおいて再度お試しください"
    )
    PhoneNumberCheckVerificationCodeRequestQuotaExceeded = "チャットをするには電話番号認証が必要です"
    GroupOfferHasBeenAccepted = "サークルの招待は承諾されています"
    GroupOfferHasBeenWithdrawn = "サークルの招待は拒否されています"
    IpBanned = "IPがBanされました"
    NotConnectedToTwitter = "Twitterに接続されていません"
    PrivateUserTimeline = "フォロワーにのみ投稿を公開しています"
    CounterRefreshLimitExceeded = "カウンター更新の回数制限に達しました"
    NotFollowedByOpponent = "このユーザーがあなたをフォローする必要があります"
    ExceedChangeCountryQuota = "国設定は一度設定すると8時間は変更できません。時間をおいて再度お試しください"
    NotGroupMember = "サークルメンバーでないため、この通話に参加できません"
    GroupPendingTransfer = "グループ保留中の移動"
    GroupPendingDeputization = "グループ保留中の代理指名"
    UserRestrictedChatWithCautionUsers = "相手は危険なユーザーとのチャットを許可していません"
    RestrictedCreateChatWithNewUsers = "近日ペナルティなどを受けているため、新規のユーザーとのチャットルーム作成が制限されています"
    RepostPostNotRepostable = "このユーザーの投稿は(´∀｀∩)↑age↑できません"
    TooManyAccountsCreated = "これ以上アカウントを作成することはできません"
    OnlySpecificGenderCanJoinGroup = "特定の性別のみ参加が許可されているサークルです"
    CreateSpecificGenderGroupRequiredGender = "このサークルを作成するには性別の設定が必要です"
    GroupRelatedExceededNumberOfRelatedGroups = "サークルの追加上限に到達しました"
    ExceededPinnedLimit = "これ以上ピン留めできません。解除してからもう一度お試しください"
    GroupShareOnTwitterLimitExceeded = "Twitterでのグループ共有の制限を超えています"
    ReportedContent = "通報を受けているため、この投稿は(´∀｀∩)↑age↑できません"
    ConferenceCallIsForMutualFollowsOnly = "この通話に参加するには枠主と相互にフォローする必要があります"
    ExceededLimit = "投稿の編集上限に達しています。これ以上の編集はできません"
    GroupInviteExceeded = "サークルにおける招待の許容数を超えました。時間をあけてから再度行ってください。"
    PhoneVerificationRequired = "参加するには電話番号認証が必要です。Yay! アプリから電話番号認証を行なってください。"
    ContentTooOld = "一定期間の経過後、投稿の編集はできません"
    PasswordTooShort = "パスワードが短すぎます"
    PasswordTooLong = "パスワードが長すぎます"
    PasswordNotAllowed = "他のパスワードを使用してください。無効な文字列が含まれています"
    CommonPassword = "他のパスワードを使用してください。文字列や数字、または記号などの組み合わせをお試しください"
    EmailNotAuthorized = "このメールアドレスを使用するには承認される必要があります"
    UnableToMovePostToThread = "投稿をスレッドに移動できません"
    UnableToPostUrl = "URLを投稿できません"
    UnableToSetCall = "通話を開始できませんでした"
    PhoneNumberBanned = "電話番号がBanされています"
    TooManyRequests = "リクエストが多すぎます"



"""
MIT License

Copyright (c) 2023 qvco

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import os
import httpx
from io import BytesIO
from PIL import Image
from urllib import parse

from .. import client
from ..config import Configs
from ..models import Attachment
from ..responses import (
    EmailGrantTokenResponse,
    EmailVerificationPresignedUrlResponse,
    PresignedUrlResponse,
    PresignedUrlsResponse,
    IdCheckerPresignedUrlResponse,
    VerifyDeviceResponse,
    WebSocketTokenResponse,
    ApplicationConfigResponse,
    BanWordsResponse,
    PopularWordsResponse,
    PolicyAgreementsResponse,
)
from ..types import ImageType
from ..utils import is_valid_image_format, get_hashed_filename, generate_uuid


upload_item_types = [
    "post",
    "chat_message",
    "chat_background",
    "report",
    "user_avatar",
    "user_cover",
    "group_cover",
    "group_thread_icon",
    "group_icon",
]


class MiscAPI(object):
    def __init__(self, base: client.BaseClient) -> None:
        self.__base = base

    def accept_policy_agreement(self, type: str):
        return self.__base._request(
            "POST",
            route=f"/v1/users/policy_agreements/{type}",
            bypass_delay=True,
        )

    def generate_sns_thumbnail(self, **params):
        """

        Parameters:
        ----------

            - resource_type: str - (Required)
            - resource_id: int - (Required)

        """
        return self.__base._request(
            "GET", route=f"/v1/sns_thumbnail/generate", params=params
        )

    def send_verification_code(self, email: str, intent: str, locale: str):
        return self.__base._request(
            "POST",
            route=self.get_email_verification_presigned_url(
                email=email, locale=locale, intent=intent
            ).url,
            payload={"locale": "ja", "email": email},
            base_url=None
        )

    def get_email_grant_token(self, code: int, email: str) -> EmailGrantTokenResponse:
        return self.__base._request(
            "POST",
            base_url=Configs.ID_CARD_CHECK_HOST_PRODUCTION,
            route="/apis/v1/apps/yay/email_grant_tokens",
            payload={"code": code, "email": email},
            data_type=EmailGrantTokenResponse,
        )

    def get_email_verification_presigned_url(
        self, email: str, locale: str, intent: str = None
    ) -> str:
        return self.__base._request(
            "POST",
            route="/v1/email_verification_urls",
            payload={
                "device_uuid": self.__base.device_uuid,
                "email": email,
                "locale": locale,
                "intent": intent,
            },
            data_type=EmailVerificationPresignedUrlResponse,
            bypass_delay=True,
        )

    def get_file_upload_presigned_urls(
        self, file_names: list[str]
    ) -> PresignedUrlsResponse:
        return self.__base._request(
            "GET",
            route="/v1/buckets/presigned_urls",
            params={"file_names[]": file_names},
            data_type=PresignedUrlsResponse,
            bypass_delay=True,
        )

    def get_id_checker_presigned_url(
        self, model: str, action: str, **params
    ) -> IdCheckerPresignedUrlResponse:
        # TODO: @QueryMap @NotNull Map<String, String> map
        """
        Meow..
        """
        return self.__base._request(
            "GET",
            route=f"/v1/id_check/{model}/{action}",
            params=params,
            data_type=IdCheckerPresignedUrlResponse,
            bypass_delay=True,
        )

    def get_old_file_upload_presigned_url(
        self, video_file_name: str
    ) -> PresignedUrlResponse:
        return self.__base._request(
            "GET",
            route=f"/v1/users/presigned_url",
            params={"video_file_name": video_file_name},
            data_type=PresignedUrlResponse,
            bypass_delay=True,
        )

    def get_policy_agreements(self) -> PolicyAgreementsResponse:
        return self.__base._request(
            "GET",
            route=f"/v1/users/policy_agreements",
            data_type=PolicyAgreementsResponse,
            bypass_delay=True,
        )

    def get_web_socket_token(self, headers: dict = None) -> WebSocketTokenResponse:
        return self.__base._request(
            "GET",
            route=f"/v1/users/ws_token",
            data_type=WebSocketTokenResponse,
            headers=headers,
            bypass_delay=True,
        )

    def verify_device(
        self,
        app_version: str,
        device_uuid: str,
        platform: str,
        verification_string: str,
    ) -> VerifyDeviceResponse:
        # TODO: check platform, verification_string
        return self.__base._request(
            "POST",
            route="/v1/genuine_devices/verify",
            payload={
                "app_version": app_version,
                "device_uuid": device_uuid,
                "platform": platform,
                "verification_string": verification_string,
            },
            data_type=VerifyDeviceResponse,
        )

    def upload_image(self, image_paths: list[str], image_type: str) -> list[Attachment]:
        """

        画像をアップロードして、サーバー上のファイル名のリストを返します。

        Parameteres
        -----------

            - image_path: list[str] - (required): 画像パスのリスト
            - image_type: str - (required): 画像の種類

        Examples
        --------

        投稿に画像を付与する場合

        >>> # サーバー上にアップロード
        >>> attachments = api.upload_image(
        >>>     image_type=yaylib.IMAGE_TYPE_POST,
        >>>     image_paths=["./test.jpg"],
        >>> )
        >>> # サーバー上のファイル名を指定
        >>> api.create_post(
        >>>     "Hello with yaylib!",
        >>>     attachment_filename=attachments[0].filename
        >>> )

        """
        if image_type not in upload_item_types:
            raise TypeError(f"Invalid image type. [{image_type}]")

        _files = []

        for key, image_path in enumerate(image_paths):
            filename, extension = os.path.splitext(image_path)

            if not is_valid_image_format(extension):
                raise ValueError(f"Invalid image format. [{filename + extension}]")

            image = Image.open(image_path)
            natural_width, natural_height = image.size

            resized_image = Image.open(image_path)

            if extension != ".gif" and image_type == ImageType.user_avatar:
                resized_image.thumbnail((200, 200))

            original_attachment = Attachment(
                file=image,
                filename="",
                original_file_name=filename,
                original_file_extension=extension,
                natural_width=natural_width,
                natural_height=natural_height,
                is_thumb=False,
            )

            thumbnail_attachment = Attachment(
                file=resized_image,
                filename="",
                original_file_name=filename,
                original_file_extension=extension,
                natural_width=natural_width,
                natural_height=natural_height,
                is_thumb=True,
            )

            uuid = generate_uuid(False)[:16]

            original_attachment.filename = get_hashed_filename(
                original_attachment, image_type, key, uuid
            )
            thumbnail_attachment.filename = get_hashed_filename(
                thumbnail_attachment, image_type, key, uuid
            )

            _files.append(original_attachment)
            _files.append(thumbnail_attachment)

        file_names = [x.filename for x in _files]
        res_presigned_url = self.get_file_upload_presigned_urls(
            file_names
        ).presigned_urls

        res_upload = []

        x: Attachment
        for x in _files:
            p_url = next(
                (p.url for p in res_presigned_url if x.filename in p.filename), None
            )
            if not p_url:
                continue

            image_data = BytesIO()
            if x.file.format == "GIF" and x.file.is_animated:
                x.file.save(image_data, format=x.file.format, save_all=True)
            else:
                x.file.save(image_data, format=x.file.format)
            image_data.seek(0)

            response = httpx.put(p_url, data=image_data.read())
            response.raise_for_status()

            x.filename = parse.urlsplit(p_url).path.replace("/uploads/", "")

            res_upload.append(x)

        return res_upload

    def upload_video(self, video_path: str):
        pass

    # config

    def get_app_config(self) -> ApplicationConfigResponse:
        return self.__base._request(
            "GET",
            route=f"https://{Configs.CONFIG_HOST}/api/apps/yay",
            data_type=ApplicationConfigResponse,
        )

    def get_banned_words(self, country_code: str = "jp") -> BanWordsResponse:
        return self.__base._request(
            "GET",
            route=f"https://{Configs.CONFIG_HOST}/{country_code}/api/v2/banned_words",
            data_type=BanWordsResponse,
        )

    def get_popular_words(self, country_code: str = "jp") -> PopularWordsResponse:
        return self.__base._request(
            "GET",
            route=f"https://{Configs.CONFIG_HOST}/{country_code}/api/apps/yay/popular_words",
            data_type=PopularWordsResponse,
        )



"""
MIT License

Copyright (c) 2023 qvco

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .models import (
    Activity,
    ApplicationConfig,
    ApplicationConfigSettings,
    BanWord,
    Bgm,
    CallGiftHistory,
    ChatRoom,
    ChatRoomDraft,
    Choice,
    CoinAmount,
    CoinExpiration,
    CoinProduct,
    CoinProductQuota,
    ConferenceCall,
    ConferenceCallUserRole,
    ContactStatus,
    CreateGroupQuota,
    TimelineSettings,
    Error,
    Footprint,
    Game,
    Genre,
    GifImage,
    GifImageCategory,
    Gift,
    GiftCount,
    GiftHistory,
    GiftingAbility,
    Group,
    GroupCategory,
    GroupGiftHistory,
    GroupUser,
    HiddenRecommendedPost,
    Interest,
    Message,
    MessageTag,
    MuteKeyword,
    PlatformDetails,
    PopularWord,
    Post,
    PostGift,
    PostTag,
    PresignedUrl,
    Promotion,
    ReceivedGift,
    RecentSearch,
    RefreshCounterRequest,
    Review,
    SearchCriteria,
    Setting,
    Settings,
    Shareable,
    SharedUrl,
    SnsInfo,
    Sticker,
    StickerPack,
    Survey,
    ThreadInfo,
    User,
    UserAuth,
    UserWrapper,
    Video,
    Walkthrough,
    WalletTransaction,
)


class ActiveFollowingsResponse:
    __slots__ = ("data", "last_loggedin_at", "users")

    def __init__(self, data):
        self.data = data
        self.last_loggedin_at = data.get("last_loggedin_at")

        self.users = data.get("users")
        if self.users is not None:
            self.users = [User(user) for user in self.users]

    def __repr__(self):
        return f"ActiveFollowingsResponse(data={self.data})"


class ActivitiesResponse:
    __slots__ = ("data", "activities", "last_timestamp")

    def __init__(self, data):
        self.data = data

        self.activities = data.get("activities")
        if self.activities is not None:
            self.activities = [Activity(activity) for activity in self.activities]

        self.last_timestamp = data.get("last_timestamp")

    def __repr__(self):
        return f"ActivitiesResponse(data={self.data})"


class AdditionalSettingsResponse:
    __slots__ = ("data", "settings")

    def __init__(self, data):
        self.data = data

        self.settings = data.get("settings")
        if self.settings is not None:
            self.settings = Settings(self.settings)

    def __repr__(self):
        return f"AdditionalSettingsResponse(data={self.data})"


class AppReviewStatusResponse:
    __slots__ = ("data", "is_app_reviewed")

    def __init__(self, data):
        self.data = data
        self.is_app_reviewed = data.get("is_app_reviewed")

    def __repr__(self):
        return f"AppReviewStatusResponse(data={self.data})"


class BgmsResponse:
    __slots__ = ("data", "bgm")

    def __init__(self, data):
        self.data = data

        self.bgm = data.get("bgm")
        if self.bgm is not None:
            self.bgm = [Bgm(bgm) for bgm in self.bgm]

    def __repr__(self):
        return f"BgmsResponse(data={self.data})"


class BlockedUserIdsResponse:
    __slots__ = ("data", "block_ids")

    def __init__(self, data):
        self.data = data

        self.block_ids = data.get("block_ids")

    def __repr__(self):
        return f"BlockedUserIdsResponse(data={self.data})"


class BlockedUsersResponse:
    __slots__ = ("data", "blocked_count", "last_id", "users")

    def __init__(self, data):
        self.data = data
        self.blocked_count = data.get("blocked_count")
        self.last_id = data.get("last_id")

        self.users = data.get("users")
        if self.users is not None:
            self.users = [User(user) for user in self.users]

    def __repr__(self):
        return f"BlockedUsersResponse(data={self.data})"


class BookmarkPostResponse:
    __slots__ = ("data", "is_bookmarked")

    def __init__(self, data):
        self.data = data
        self.is_bookmarked = data.get("is_bookmarked")

    def __repr__(self):
        return f"BookmarkPostResponse(data={self.data})"


class CallStatusResponse:
    __slots__ = ("data", "phone_status", "video_status", "room_url")

    def __init__(self, data):
        self.data = data
        self.phone_status = data.get("phone_status")
        self.video_status = data.get("video_status")
        self.room_url = data.get("room_url")

    def __repr__(self):
        return f"CallStatusResponse(data={self.data})"


class ChatRoomResponse:
    __slots__ = ("data", "chat")

    def __init__(self, data):
        self.data = data

        self.chat = data.get("chat")
        if self.chat is not None:
            self.chat = ChatRoom(self.chat)

    def __repr__(self):
        return f"ChatRoomResponse(data={self.data})"


class ChatRoomsResponse:
    __slots__ = ("data", "pinned_chat_rooms", "chat_rooms", "next_page_value")

    def __init__(self, data):
        self.data = data

        self.pinned_chat_rooms = data.get("pinned_chat_rooms")
        if self.pinned_chat_rooms is not None:
            self.pinned_chat_rooms = [
                ChatRoom(pinned_chat_room)
                for pinned_chat_room in self.pinned_chat_rooms
            ]

        self.chat_rooms = data.get("chat_rooms")
        if self.chat_rooms is not None:
            self.chat_rooms = [ChatRoom(chat_room) for chat_room in self.chat_rooms]

        self.next_page_value = data.get("next_page_value")

    def __repr__(self):
        return f"ChatRoomsResponse(data={self.data})"


class TotalChatRequestResponse:
    __slots__ = ("data", "total")

    def __init__(self, data):
        self.data = data
        self.total = data.get("total")

    def __repr__(self):
        return f"TotalChatRequestResponse(data={self.data})"


class ConferenceCallResponse:
    __slots__ = ("data", "conference_call")

    def __init__(self, data):
        self.data = data

        self.conference_call = data.get("conference_call")
        if self.conference_call is not None:
            self.conference_call = ConferenceCall(self.conference_call)

    def __repr__(self):
        return f"ConferenceCallResponse(data={self.data})"


class ContactStatusResponse:
    __slots__ = ("data", "contacts")

    def __init__(self, data):
        self.data = data
        self.contacts = data.get("contacts")

    def __repr__(self):
        return f"ContactStatusResponse(data={self.data})"


class CreateGroupResponse:
    __slots__ = ("data", "group_id")

    def __init__(self, data):
        self.data = data
        self.group_id = data.get("group_id")

    def __repr__(self):
        return f"CreateGroupResponse(data={self.data})"


class CreateQuotaResponse:
    __slots__ = ("data", "create")

    def __init__(self, data):
        self.data = data

        self.create = data.get("create")
        if self.create is not None:
            self.create = CreateGroupQuota(self.create)

    def __repr__(self):
        return f"CreateQuotaResponse(data={self.data})"


class CreateChatRoomResponse:
    __slots__ = ("data", "room_id")

    def __init__(self, data):
        self.data = data
        self.room_id = data.get("room_id")

    def __repr__(self):
        return f"CreateChatRoomResponse(data={self.data})"


class CreatePostResponse:
    __slots__ = ("data", "conference_call", "post")

    def __init__(self, data):
        self.data = data

        self.conference_call = data.get("conference_call")
        if self.conference_call is not None:
            self.conference_call = ConferenceCall(self.conference_call)

        self.post = data.get("post")
        if self.post is not None:
            self.post = Post(self.post)

    def __repr__(self):
        return f"CreatePostResponse(data={self.data})"


class CreateUserResponse:
    __slots__ = ("data", "id", "access_token", "refresh_token", "expires_in")

    def __init__(self, data):
        self.data = data
        self.id = data.get("id")
        self.access_token = data.get("access_token")
        self.refresh_token = data.get("refresh_token")
        self.expires_in = data.get("expires_in")

    def __repr__(self):
        return f"CreateUserResponse(data={self.data})"


class EmailGrantTokenResponse:
    __slots__ = ("data", "email_grant_token")

    def __init__(self, data):
        self.data = data
        self.email_grant_token = data.get("email_grant_token")

    def __repr__(self):
        return f"EmailGrantTokenResponse(data={self.data})"


class EmailVerificationPresignedUrlResponse:
    __slots__ = ("data", "url", "presigned_url", "expires_at", "method_type")

    def __init__(self, data):
        self.data = data
        self.url = data.get("url")
        self.presigned_url = data.get("presigned_url")
        self.expires_at = data.get("expires_at")
        self.method_type = data.get("method_type")

    def __repr__(self):
        return f"EmailVerificationPresignedUrlResponse(data={self.data})"


class PresignedUrlResponse:
    __slots__ = ("data", "presigned_url")

    def __init__(self, data):
        self.data = data
        self.presigned_url = data.get("presigned_url")

    def __repr__(self):
        return f"PresignedUrlResponse(data={self.data})"


class PresignedUrlsResponse:
    __slots__ = ("data", "presigned_urls")

    def __init__(self, data):
        self.data = data

        self.presigned_urls = data.get("presigned_urls")
        if self.presigned_urls is not None:
            self.presigned_urls = [
                PresignedUrl(presigned_url) for presigned_url in self.presigned_urls
            ]

    def __repr__(self):
        return f"PresignedUrlsResponse(data={self.data})"


class IdCheckerPresignedUrlResponse:
    __slots__ = ("data", "presigned_url")

    def __init__(self, data):
        self.data = data
        self.presigned_url = data.get("presigned_url")

    def __repr__(self):
        return f"IdCheckerPresignedUrlResponse(data={self.data})"


class DefaultSettingsResponse:
    __slots__ = ("data", "timeline_settings")

    def __init__(self, data):
        self.data = data

        self.timeline_settings = data.get("timeline_settings")
        if self.timeline_settings is not None:
            self.timeline_settings = TimelineSettings(self.timeline_settings)

    def __repr__(self):
        return f"DefaultSettingsResponse(data={self.data})"


class FollowRecommendationsResponse:
    __slots__ = ("data", "total", "users", "next")

    def __init__(self, data):
        self.data = data
        self.total = data.get("total")

        self.users = data.get("users")
        if self.users is not None:
            self.users = [User(user) for user in self.users]

        self.next = data.get("next")

    def __repr__(self):
        return f"FollowRecommendationsResponse(data={self.data})"


class FollowRequestCountResponse:
    __slots__ = ("data", "users_count")

    def __init__(self, data):
        self.data = data
        self.users_count = data.get("users_count")

    def __repr__(self):
        return f"FollowRequestCountResponse(data={self.data})"


class FollowUsersResponse:
    __slots__ = ("data", "last_follow_id", "users")

    def __init__(self, data):
        self.data = data
        self.last_follow_id = data.get("last_follow_id")

        self.users = data.get("users")
        if self.users is not None:
            self.users = [User(user) for user in self.users]

    def __repr__(self):
        return f"FollowUsersResponse(data={self.data})"


class FootprintsResponse:
    __slots__ = ("data", "footprints")

    def __init__(self, data):
        self.data = data

        self.footprints = data.get("footprints")
        if self.footprints is not None:
            self.footprints = [Footprint(footprint) for footprint in self.footprints]

    def __repr__(self):
        return f"FootprintsResponse(data={self.data})"


class GamesResponse:
    __slots__ = ("data", "games", "from_id")

    def __init__(self, data):
        self.data = data

        self.games = data.get("games")
        if self.games is not None:
            self.games = [Game(game) for game in self.games]

        self.from_id = data.get("from_id")

    def __repr__(self):
        return f"GamesResponse(data={self.data})"


class GenresResponse:
    __slots__ = ("data", "genres", "next_page_value")

    def __init__(self, data):
        self.data = data

        self.genres = data.get("genres")
        if self.genres is not None:
            self.genres = [Genre(genre) for genre in self.genres]

        self.next_page_value = data.get("next_page_value")

    def __repr__(self):
        return f"GenresResponse(data={self.data})"


class GroupCategoriesResponse:
    __slots__ = ("data", "group_categories")

    def __init__(self, data):
        self.data = data

        self.group_categories = data.get("group_categories")
        if self.group_categories is not None:
            self.group_categories = [
                GroupCategory(group_categorie)
                for group_categorie in self.group_categories
            ]

    def __repr__(self):
        return f"GroupCategoriesResponse(data={self.data})"


class GroupGiftHistoryResponse:
    __slots__ = ("data", "gift_history", "next_page_value")

    def __init__(self, data):
        self.data = data

        self.gift_history = data.get("gift_history")
        if self.gift_history is not None:
            self.gift_history = [
                GroupGiftHistory(gift_history) for gift_history in self.gift_history
            ]

        self.next_page_value = data.get("next_page_value")

    def __repr__(self):
        return f"GroupGiftHistoryResponse(data={self.data})"


class GroupNotificationSettingsResponse:
    __slots__ = ("data", "setting")

    def __init__(self, data):
        self.data = data

        self.setting = data.get("setting")
        if self.setting is not None:
            self.setting = Setting(self.setting)

    def __repr__(self):
        return f"GroupNotificationSettingsResponse(data={self.data})"


class GroupResponse:
    __slots__ = ("data", "group")

    def __init__(self, data):
        self.data = data

        self.group = data.get("group")
        if self.group is not None:
            self.group = Group(self.group)

    def __repr__(self):
        return f"GroupResponse(data={self.data})"


class GroupsRelatedResponse:
    __slots__ = ("data", "groups", "next_page_value")

    def __init__(self, data):
        self.data = data

        self.groups = data.get("groups")
        if self.groups is not None:
            self.groups = [Group(group) for group in self.groups]

        self.next_page_value = data.get("next_page_value")

    def __repr__(self):
        return f"GroupsRelatedResponse(data={self.data})"


class GroupsResponse:
    __slots__ = ("data", "pinned_groups", "groups")

    def __init__(self, data):
        self.data = data

        self.pinned_groups = data.get("pinned_groups")
        if self.pinned_groups is not None:
            self.pinned_groups = [
                Group(pinned_group) for pinned_group in self.pinned_groups
            ]

        self.groups = data.get("groups")
        if self.groups is not None:
            self.groups = [Group(group) for group in self.groups]

    def __repr__(self):
        return f"GroupsResponse(data={self.data})"


class GroupThreadListResponse:
    __slots__ = ("data", "threads", "next_page_value")

    def __init__(self, data):
        self.data = data

        self.threads = data.get("threads")
        if self.threads is not None:
            self.threads = [ThreadInfo(thread) for thread in self.threads]

        self.next_page_value = data.get("next_page_value")

    def __repr__(self):
        return f"GroupThreadListResponse(data={self.data})"


class GroupUserResponse:
    __slots__ = ("data", "group_user")

    def __init__(self, data):
        self.data = data

        self.group_user = data.get("group_user")
        if self.group_user is not None:
            self.group_user = GroupUser(self.group_user)

    def __repr__(self):
        return f"GroupUserResponse(data={self.data})"


class GroupUsersResponse:
    __slots__ = ("data", "group_users")

    def __init__(self, data):
        self.data = data

        self.group_users = data.get("group_users")
        if self.group_users is not None:
            self.group_users = [
                GroupUser(group_user) for group_user in self.group_users
            ]

    def __repr__(self):
        return f"GroupUsersResponse(data={self.data})"


class GifsDataResponse:
    __slots__ = ("data", "gif_categories")

    def __init__(self, data):
        self.data = data

        self.gif_categories = data.get("gif_categories")
        if self.gif_categories is not None:
            self.gif_categories = [
                GifImageCategory(gif_category) for gif_category in self.gif_categories
            ]

    def __repr__(self):
        return f"GifsDataResponse(data={self.data})"


class HiddenResponse:
    __slots__ = ("data", "hidden_users", "next_page_value", "total_count", "limit")

    def __init__(self, data):
        self.data = data

        self.hidden_users = data.get("hidden_users")
        if self.hidden_users is not None:
            self.hidden_users = [User(hidden_user) for hidden_user in self.hidden_users]

        self.next_page_value = data.get("next_page_value")
        self.total_count = data.get("total_count")
        self.limit = data.get("limit")

    def __repr__(self):
        return f"HiddenResponse(data={self.data})"


class LoginUserResponse:
    __slots__ = (
        "data",
        "user_id",
        "username",
        "is_new",
        "sns_info",
        "access_token",
        "refresh_token",
        "expires_in",
    )

    def __init__(self, data):
        self.data = data
        self.user_id = data.get("user_id")
        self.username = data.get("username")
        self.is_new = data.get("is_new")

        self.sns_info = data.get("sns_info")
        if self.sns_info is not None:
            self.sns_info = SnsInfo(self.sns_info)

        self.access_token = data.get("access_token")
        self.refresh_token = data.get("refresh_token")
        self.expires_in = data.get("expires_in")

    def __repr__(self):
        return f"LoginUserResponse(data={self.data})"


class LoginUpdateResponse:
    __slots__ = (
        "data",
        "user_id",
        "username",
        "access_token",
        "refresh_token",
        "expires_in",
    )

    def __init__(self, data):
        self.data = data
        self.user_id = data.get("user_id")
        self.username = data.get("username")
        self.access_token = data.get("access_token")
        self.refresh_token = data.get("refresh_token")
        self.expires_in = data.get("expires_in")

    def __repr__(self):
        return f"LoginUpdateResponse(data={self.data})"


class MessageResponse:
    __slots__ = ("data", "id", "conference_call")

    def __init__(self, data):
        self.data = data
        self.id = data.get("id")

        self.conference_call = data.get("conference_call")
        if self.conference_call is not None:
            self.conference_call = ConferenceCall(self.conference_call)

    def __repr__(self):
        return f"MessageResponse(data={self.data})"


class MessagesResponse:
    __slots__ = ("data", "messages")

    def __init__(self, data):
        self.data = data

        self.messages = data.get("messages")
        if self.messages is not None:
            self.messages = [Message(message) for message in self.messages]

    def __repr__(self):
        return f"MessagesResponse(data={self.data})"


class PolicyAgreementsResponse:
    __slots__ = ("data", "latest_privacy_policy_agreed", "latest_terms_of_use_agreed")

    def __init__(self, data):
        self.data = data
        self.latest_privacy_policy_agreed = data.get("latest_privacy_policy_agreed")
        self.latest_terms_of_use_agreed = data.get("latest_terms_of_use_agreed")

    def __repr__(self):
        return f"PolicyAgreementsResponse(data={self.data})"


class PostResponse:
    __slots__ = ("data", "post")

    def __init__(self, data):
        self.data = data

        self.post = data.get("post")
        if self.post is not None:
            self.post = Post(self.post)

    def __repr__(self):
        return f"PostResponse(data={self.data})"


class PostsResponse:
    __slots__ = ("data", "next_page_value", "posts", "pinned_posts")

    def __init__(self, data):
        self.data = data
        self.next_page_value = data.get("next_page_value")

        self.posts = data.get("posts")
        if self.posts is not None:
            self.posts = [Post(post) for post in self.posts]

        self.pinned_posts = data.get("pinned_posts")
        if self.pinned_posts is not None:
            self.pinned_posts = [Post(pinned_post) for pinned_post in self.pinned_posts]

    def __repr__(self):
        return f"PostsResponse(data={self.data})"


class PostLikersResponse:
    __slots__ = ("data", "last_id", "users")

    def __init__(self, data):
        self.data = data
        self.last_id = data.get("last_id")

        self.users = data.get("users")
        if self.users is not None:
            self.users = [User(user) for user in self.users]

    def __repr__(self):
        return f"PostLikersResponse(data={self.data})"


class PostTagsResponse:
    __slots__ = ("data", "tags")

    def __init__(self, data):
        self.data = data

        self.tags = data.get("tags")
        if self.tags is not None:
            self.tags = [PostTag(tag) for tag in self.tags]

    def __repr__(self):
        return f"PostTagsResponse(data={self.data})"


class PromotionsResponse:
    __slots__ = ("data", "promotions")

    def __init__(self, data):
        self.data = data

        self.promotions = data.get("promotions")
        if self.promotions is not None:
            self.promotions = [Promotion(promotion) for promotion in self.promotions]

    def __repr__(self):
        return f"PromotionsResponse(data={self.data})"


class LikePostsResponse:
    __slots__ = ("data", "like_ids")

    def __init__(self, data):
        self.data = data
        self.like_ids = data.get("like_ids")

    def __repr__(self):
        return f"LikePostsResponse(data={self.data})"


class ValidationPostResponse:
    __slots__ = ("data", "is_allow_to_post")

    def __init__(self, data):
        self.data = data
        self.is_allow_to_post = data.get("is_allow_to_post")

    def __repr__(self):
        return f"ValidationPostResponse(data={self.data})"


class RefreshCounterRequestsResponse:
    __slots__ = ("data", "reset_counter_requests")

    def __init__(self, data):
        self.data = data

        self.reset_counter_requests = data.get("reset_counter_requests")
        if self.reset_counter_requests is not None:
            self.reset_counter_requests = [
                RefreshCounterRequest(reset_counter_request)
                for reset_counter_request in self.reset_counter_requests
            ]

    def __repr__(self):
        return f"RefreshCounterRequestsResponse(data={self.data})"


class RegisterDeviceTokenResponse:
    __slots__ = ("data", "id", "server_device_id", "uuid", "created_at", "updated_at")

    def __init__(self, data):
        self.data = data
        self.id = data.get("id")
        self.server_device_id = data.get("server_device_id")
        self.uuid = data.get("uuid")
        self.created_at = data.get("created_at")
        self.updated_at = data.get("updated_at")

    def __repr__(self):
        return f"RegisterDeviceTokenResponse(data={self.data})"


class ReviewsResponse:
    __slots__ = ("data", "reviews", "pinned_reviews")

    def __init__(self, data):
        self.data = data

        self.reviews = data.get("reviews")
        if self.reviews is not None:
            self.reviews = [Review(review) for review in self.reviews]

        self.pinned_reviews = data.get("pinned_reviews")
        if self.pinned_reviews is not None:
            self.pinned_reviews = [
                Review(pinned_review) for pinned_review in self.pinned_reviews
            ]

    def __repr__(self):
        return f"ReviewsResponse(data={self.data})"


class SocialShareUsersResponse:
    __slots__ = ("data", "social_shared_users")

    def __init__(self, data):
        self.data = data

        self.social_shared_users = data.get("social_shared_users")
        if self.social_shared_users is not None:
            self.social_shared_users = [
                UserWrapper(social_shared_user)
                for social_shared_user in self.social_shared_users
            ]

    def __repr__(self):
        return f"SocialShareUsersResponse(data={self.data})"


class StickerPacksResponse:
    __slots__ = ("data", "sticker_packs")

    def __init__(self, data):
        self.data = data

        self.sticker_packs = data.get("sticker_packs")
        if self.sticker_packs is not None:
            self.sticker_packs = [
                StickerPack(sticker_pack) for sticker_pack in self.sticker_packs
            ]

    def __repr__(self):
        return f"StickerPacksResponse(data={self.data})"


class TokenResponse:
    __slots__ = (
        "data",
        "user_id",
        "created_at",
        "access_token",
        "refresh_token",
        "expires_in",
    )

    def __init__(self, data):
        self.data = data
        self.user_id = data.get("id")
        self.created_at = data.get("created_at")
        self.access_token = data.get("access_token")
        self.refresh_token = data.get("refresh_token")
        self.expires_in = data.get("expires_in")

    def __repr__(self):
        return f"TokenResponse(data={self.data})"


class VerifyDeviceResponse:
    __slots__ = ("data", "verified", "verified_at")

    def __init__(self, data):
        self.data = data
        self.verified = data.get("verified")
        self.verified_at = data.get("verified_at")

    def __repr__(self):
        return f"VerifyDeviceResponse(data={self.data})"


class VipGameRewardUrlResponse:
    __slots__ = ("data", "url")

    def __init__(self, data):
        self.data = data
        self.url = data.get("url")

    def __repr__(self):
        return f"VipGameRewardUrlResponse(data={self.data})"


class VoteSurveyResponse:
    __slots__ = ("data", "survey")

    def __init__(self, data):
        self.data = data

        self.survey = data.get("survey")
        if self.survey is not None:
            self.survey = Survey(self.survey)

    def __repr__(self):
        return f"VoteSurveyResponse(data={self.data})"


class UnreadStatusResponse:
    __slots__ = ("data", "is_unread")

    def __init__(self, data):
        self.data = data
        self.is_unread = data.get("is_unread")

    def __repr__(self):
        return f"UnreadStatusResponse(data={self.data})"


class UserResponse:
    __slots__ = (
        "data",
        "user",
        "masked_email",
        "twitter_id",
        "is_line_connected",
        "is_facebook_connected",
        "is_lobi_connected",
        "is_push_notification_on",
        "is_call_on",
        "is_video_on",
        "is_group_call_on",
        "is_group_video_on",
        "vip_until",
        "is_email_confirmed",
        "blocking_limit",
        "uuid",
        "birthdate",
        "gifting_ability",
    )

    def __init__(self, data):
        self.data = data

        self.user = data.get("user")
        if self.user is not None:
            self.user = User(self.user)

        self.masked_email = data.get("masked_email")
        self.twitter_id = data.get("twitter_id")
        self.is_line_connected = data.get("line_connected")
        self.is_facebook_connected = data.get("facebook_connected")
        self.is_lobi_connected = data.get("lobi_connected")
        self.is_push_notification_on = data.get("push_notification")
        self.is_call_on = data.get("phone_on")
        self.is_video_on = data.get("video_on")
        self.is_group_call_on = data.get("group_phone_on")
        self.is_group_video_on = data.get("group_video_on")
        self.vip_until = data.get("vip_until_seconds")
        self.is_email_confirmed = data.get("email_confirmed")
        self.blocking_limit = data.get("blocking_limit")
        self.uuid = data.get("uuid")
        self.birthdate = data.get("birth_date")

        self.gifting_ability = data.get("gifting_ability")
        if self.gifting_ability is not None:
            self.gifting_ability = GiftingAbility(self.gifting_ability)

    def __repr__(self):
        return f"UserResponse(data={self.data})"


class UsersResponse:
    __slots__ = ("data", "users", "next_page_value")

    def __init__(self, data):
        self.data = data

        self.users = data.get("users")
        if self.users is not None:
            self.users = [User(user) for user in self.users]

        self.next_page_value = data.get("next_page_value")

    def __repr__(self):
        return f"UsersResponse(data={self.data})"


class RankingUsersResponse:
    __slots__ = ("data", "users")

    def __init__(self, data):
        self.data = data

        self.users = data.get("users")
        if self.users is not None:
            self.users = [User(user) for user in self.users]

    def __repr__(self):
        return f"RankingUsersResponse(data={self.data})"


class UserCustomDefinitionsResponse:
    __slots__ = (
        "age",
        "followers_count",
        "followings_count",
        "created_at",
        "last_loggedin_at",
        "status",
        "reported_count",
    )

    def __init__(self, data):
        self.data = data
        self.age = data.get("age")
        self.followers_count = data.get("followers_count")
        self.followings_count = data.get("followings_count")
        self.created_at = data.get("created_at")
        self.last_loggedin_at = data.get("last_loggedin_at")
        self.status = data.get("status")
        self.reported_count = data.get("reported_count")

    def __repr__(self):
        return f"UserCustomDefinitionsResponse(data={self.data})"


class UserEmailResponse:
    __slots__ = ("data", "email")

    def __init__(self, data):
        self.data = data
        self.email = data.get("email")

    def __repr__(self):
        return f"UserEmailResponse(data={self.data})"


class HimaUsersResponse:
    __slots__ = ("data", "hima_users")

    def __init__(self, data):
        self.data = data

        self.hima_users = data.get("hima_users")
        if self.hima_users is not None:
            self.hima_users = [UserWrapper(hima_user) for hima_user in self.hima_users]

    def __repr__(self):
        return f"HimaUsersResponse(data={self.data})"


class UsersByTimestampResponse:
    __slots__ = ("data", "last_timestamp", "users")

    def __init__(self, data):
        self.data = data
        self.last_timestamp = data.get("last_timestamp")

        self.users = data.get("users")
        if self.users is not None:
            self.users = [User(user) for user in self.users]

    def __repr__(self):
        return f"UsersByTimestampResponse(data={self.data})"


class UserTimestampResponse:
    __slots__ = ("data", "time", "ip_address", "country")

    def __init__(self, data):
        self.data = data
        self.time = data.get("time")
        self.ip_address = data.get("ip_address")
        self.country = data.get("country")

    def __repr__(self):
        return f"UserTimestampResponse(data={self.data})"


class ChannelResponse:
    __slots__ = ("data", "identifier", "message", "type", "sid")

    def __init__(self, data):
        self.data = data
        self.identifier = data.get("identifier")

        self.message = data.get("message")
        if self.message is not None and isinstance(self.message, dict):
            self.message = WebSocketMessageResponse(self.message)

        self.type = data.get("type")
        self.sid = data.get("sid")

    def __repr__(self):
        return f"ChannelResponse(data={self.data})"


class WebSocketMessageResponse:
    __slots__ = ("response", "data", "event")

    def __init__(self, data):
        self.response = data
        self.data = data.get("data")
        self.event = data.get("event")

    def __repr__(self):
        return f"WebSocketMessageResponse(response={self.response})"


class WebSocketTokenResponse:
    __slots__ = ("data", "token")

    def __init__(self, data):
        self.data = data
        self.token = data.get("token")

    def __repr__(self):
        return f"WebSocketTokenResponse(data={self.data})"


class ApplicationConfigResponse:
    __slots__ = ("data", "app")

    def __init__(self, data):
        self.data = data

        self.app = data.get("app")
        if self.app is not None:
            self.app = ApplicationConfig(self.app)

    def __repr__(self):
        return f"ApplicationConfigResponse(data={self.data})"


class BanWordsResponse:
    __slots__ = ("data", "ban_words")

    def __init__(self, data):
        self.data = data

        self.ban_words = data.get("ban_words")
        if self.ban_words is not None:
            self.ban_words = [BanWord(ban_word) for ban_word in self.ban_words]

    def __repr__(self):
        return f"BanWordsResponse(data={self.data})"


class PopularWordsResponse:
    __slots__ = ("data", "popular_words")

    def __init__(self, data):
        self.data = data

        self.popular_words = data.get("popular_words")
        if self.popular_words is not None:
            self.popular_words = [
                PopularWord(popular_word) for popular_word in self.popular_words
            ]

    def __repr__(self):
        return f"PopularWordsResponse(data={self.data})"



"""
MIT License

Copyright (c) 2023 qvco

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

from datetime import datetime

from .. import client
from ..config import Configs
from ..errors import ForbiddenError, NotFoundError
from ..models import Post, SharedUrl
from ..responses import (
    BookmarkPostResponse,
    CreatePostResponse,
    PostResponse,
    PostsResponse,
    PostLikersResponse,
    PostTagsResponse,
    LikePostsResponse,
    VoteSurveyResponse,
)
from ..utils import build_message_tags, get_post_type, md5


class PostAPI(object):
    def __init__(self, base: client.BaseClient) -> None:
        self.__base = base

    def add_bookmark(self, user_id: int, post_id: int) -> BookmarkPostResponse:
        return self.__base._request(
            "PUT",
            route=f"/v1/users/{user_id}/bookmarks/{post_id}",
            data_type=BookmarkPostResponse,
        )

    def add_group_highlight_post(self, group_id: int, post_id: int):
        """

        投稿をグループのまとめに追加します

        """
        return self.__base._request(
            "PUT",
            route=f"/v1/groups/{group_id}/highlights/{post_id}",
        )

    def create_call_post(
        self,
        text: str = None,
        font_size: int = None,
        color: int = None,
        group_id: int = None,
        call_type: str = None,
        category_id: int = None,
        game_title: str = None,
        joinable_by: str = None,
        message_tags: list = [],
        attachment_filename: str = None,
        attachment_2_filename: str = None,
        attachment_3_filename: str = None,
        attachment_4_filename: str = None,
        attachment_5_filename: str = None,
        attachment_6_filename: str = None,
        attachment_7_filename: str = None,
        attachment_8_filename: str = None,
        attachment_9_filename: str = None,
    ) -> CreatePostResponse:
        text, message_tags = build_message_tags(text)

        return self.__base._request(
            "POST",
            route=f"/v1/posts/new_conference_call",
            payload={
                "text": text,
                "font_size": font_size,
                "color": color,
                "group_id": group_id,
                "call_type": call_type,
                "uuid": self.__base.uuid,
                "api_key": Configs.API_KEY,
                "timestamp": int(datetime.now().timestamp()),
                "signed_info": self.__signed_info,
                "category_id": category_id,
                "game_title": game_title,
                "joinable_by": joinable_by,
                "message_tags": message_tags,
                "attachment_filename": attachment_filename,
                "attachment_2_filename": attachment_2_filename,
                "attachment_3_filename": attachment_3_filename,
                "attachment_4_filename": attachment_4_filename,
                "attachment_5_filename": attachment_5_filename,
                "attachment_6_filename": attachment_6_filename,
                "attachment_7_filename": attachment_7_filename,
                "attachment_8_filename": attachment_8_filename,
                "attachment_9_filename": attachment_9_filename,
            },
            data_type=CreatePostResponse,
        )

    def create_group_pin_post(self, post_id: int, group_id: int):
        return self.__base._request(
            "PUT",
            route=f"/v2/posts/group_pinned_post",
            payload={"post_id": post_id, "group_id": group_id},
        )

    def create_pin_post(self, post_id: int):
        return self.__base._request(
            "POST", route=f"/v1/pinned/posts", payload={"id": post_id}
        )

    def create_post(
        self,
        text: str = None,
        font_size: int = 0,
        color: int = 0,
        in_reply_to: int = None,
        group_id: int = None,
        mention_ids: list[int] = None,
        choices: list[str] = None,
        shared_url: str = None,
        message_tags: list = [],
        attachment_filename: str = None,
        attachment_2_filename: str = None,
        attachment_3_filename: str = None,
        attachment_4_filename: str = None,
        attachment_5_filename: str = None,
        attachment_6_filename: str = None,
        attachment_7_filename: str = None,
        attachment_8_filename: str = None,
        attachment_9_filename: str = None,
        video_file_name: str = None,
    ) -> Post:
        result = build_message_tags(text)
        if result is not None:
            text, message_tags = result

        post_type = get_post_type(
            choices=choices,
            shared_url=shared_url,
            video_file_name=video_file_name,
            attachment_filename=attachment_filename,
        )

        if shared_url is not None:
            try:
                shared_url = self.get_url_metadata(url=shared_url).data
            except ForbiddenError:
                self.__base.logger.error("Unable to get the URL metadata.")
                shared_url = None

        return self.__base._request(
            "POST",
            route=f"/v3/posts/new",
            payload={
                "text": text,
                "font_size": font_size,
                "color": color,
                "in_reply_to": in_reply_to,
                "group_id": group_id,
                "post_type": post_type,
                "mention_ids": mention_ids,
                "choices": choices,
                "shared_url": shared_url,
                "message_tags": message_tags,
                "attachment_filename": attachment_filename,
                "attachment_2_filename": attachment_2_filename,
                "attachment_3_filename": attachment_3_filename,
                "attachment_4_filename": attachment_4_filename,
                "attachment_5_filename": attachment_5_filename,
                "attachment_6_filename": attachment_6_filename,
                "attachment_7_filename": attachment_7_filename,
                "attachment_8_filename": attachment_8_filename,
                "attachment_9_filename": attachment_9_filename,
                "video_file_name": video_file_name,
            },
            data_type=Post,
            jwt_required=True,
        )

    def create_repost(
        self,
        post_id: int,
        text: str = None,
        font_size: int = None,
        color: int = None,
        in_reply_to: int = None,
        group_id: int = None,
        mention_ids: list[int] = None,
        choices: list[str] = None,
        shared_url: dict[str, str | int] = None,
        message_tags: list = [],
        attachment_filename: str = None,
        attachment_2_filename: str = None,
        attachment_3_filename: str = None,
        attachment_4_filename: str = None,
        attachment_5_filename: str = None,
        attachment_6_filename: str = None,
        attachment_7_filename: str = None,
        attachment_8_filename: str = None,
        attachment_9_filename: str = None,
        video_file_name: str = None,
    ) -> CreatePostResponse:
        result = build_message_tags(text)
        if result is not None:
            text, message_tags = result

        post_type = get_post_type(
            choices=choices,
            shared_url=shared_url,
            video_file_name=video_file_name,
            attachment_filename=attachment_filename,
        )

        if shared_url is not None:
            try:
                shared_url = self.get_url_metadata(url=shared_url).data
            except ForbiddenError:
                self.__base.logger.error("Unable to get the URL metadata.")
                shared_url = None

        return self.__base._request(
            "POST",
            route=f"/v3/posts/repost",
            payload={
                "post_id": post_id,
                "text": text,
                "font_size": font_size,
                "color": color,
                "in_reply_to": in_reply_to,
                "group_id": group_id,
                "post_type": post_type,
                "mention_ids": mention_ids,
                "choices": choices,
                "shared_url": shared_url,
                "message_tags": message_tags,
                "attachment_filename": attachment_filename,
                "attachment_2_filename": attachment_2_filename,
                "attachment_3_filename": attachment_3_filename,
                "attachment_4_filename": attachment_4_filename,
                "attachment_5_filename": attachment_5_filename,
                "attachment_6_filename": attachment_6_filename,
                "attachment_7_filename": attachment_7_filename,
                "attachment_8_filename": attachment_8_filename,
                "attachment_9_filename": attachment_9_filename,
                "video_file_name": video_file_name,
            },
            data_type=CreatePostResponse,
            jwt_required=True,
        )

    def create_share_post(
        self,
        shareable_type: str,
        shareable_id: int,
        text: str = None,
        font_size: int = None,
        color: int = None,
        group_id: int = None,
    ) -> Post:
        return self.__base._request(
            "POST",
            route=f"/v2/posts/new_share_post",
            payload={
                "shareable_type": shareable_type,
                "shareable_id": shareable_id,
                "text": text,
                "font_size": font_size,
                "color": color,
                "group_id": group_id,
                "uuid": self.__base.uuid,
                "api_key": Configs.API_KEY,
                "timestamp": int(datetime.now().timestamp()),
                "signed_info": self.__signed_info,
            },
            data_type=Post,
        )

    def create_thread_post(
        self,
        post_id: int,
        text: str = None,
        font_size: int = None,
        color: int = None,
        in_reply_to: int = None,
        group_id: int = None,
        mention_ids: list[int] = None,
        choices: list[str] = None,
        shared_url: dict[str, str | int] = None,
        message_tags: list = [],
        attachment_filename: str = None,
        attachment_2_filename: str = None,
        attachment_3_filename: str = None,
        attachment_4_filename: str = None,
        attachment_5_filename: str = None,
        attachment_6_filename: str = None,
        attachment_7_filename: str = None,
        attachment_8_filename: str = None,
        attachment_9_filename: str = None,
        video_file_name: str = None,
    ) -> Post:
        result = build_message_tags(text)
        if result is not None:
            text, message_tags = result

        post_type = get_post_type(
            choices=choices,
            shared_url=shared_url,
            video_file_name=video_file_name,
            attachment_filename=attachment_filename,
        )

        if shared_url is not None:
            try:
                shared_url = self.get_url_metadata(url=shared_url).data
            except ForbiddenError:
                self.__base.logger.error("Unable to get the URL metadata.")
                shared_url = None

        return self.__base._request(
            "POST",
            route=f"/v1/threads/{post_id}/posts",
            payload={
                "id": post_id,
                "text": text,
                "font_size": font_size,
                "color": color,
                "in_reply_to": in_reply_to,
                "group_id": group_id,
                "post_type": post_type,
                "mention_ids": mention_ids,
                "choices": choices,
                "shared_url": shared_url,
                "message_tags": message_tags,
                "attachment_filename": attachment_filename,
                "attachment_2_filename": attachment_2_filename,
                "attachment_3_filename": attachment_3_filename,
                "attachment_4_filename": attachment_4_filename,
                "attachment_5_filename": attachment_5_filename,
                "attachment_6_filename": attachment_6_filename,
                "attachment_7_filename": attachment_7_filename,
                "attachment_8_filename": attachment_8_filename,
                "attachment_9_filename": attachment_9_filename,
                "video_file_name": video_file_name,
            },
            data_type=Post,
            jwt_required=True,
        )

    def delete_all_post(self):
        try:
            return self.__base._request("POST", route=f"/v1/posts/delete_all_post")
        except NotFoundError:
            self.__base.logger.info("Post not found. Skipping...")

    def delete_group_pin_post(self, group_id: int):
        return self.__base._request(
            "DELETE",
            route=f"/v2/posts/group_pinned_post",
            payload={"group_id": group_id},
        )

    def delete_pin_post(self, post_id: int):
        return self.__base._request("DELETE", route=f"/v1/pinned/posts/{post_id}")

    def get_bookmark(self, user_id: int, from_str: str = None) -> PostsResponse:
        params = {}
        if from_str:
            params = {"from": from_str}
        return self.__base._request(
            "GET",
            route=f"/v1/users/{user_id}/bookmarks",
            params=params,
            data_type=PostsResponse,
        )

    def get_timeline_calls(self, **params) -> PostsResponse:
        """

        Parameters:
        ----------

            - group_id: int = None
            - from_timestamp: int = None
            - number: int = None
            - category_id: int = None
            - call_type: str = "voice"
            - include_circle_call: bool = None
            - cross_generation: bool = None
            - exclude_recent_gomimushi: bool = None
            - shared_interest_categories: bool = None

        """
        return self.__base._request(
            "GET",
            route=f"/v2/posts/call_timeline",
            params=params,
            data_type=PostsResponse,
        )

    def get_conversation(self, conversation_id: int, **params) -> PostsResponse:
        """

        Parameters:
        ----------

            - conversation_id: int
            - group_id: int = None
            - thread_id: int = None
            - from_post_id: int = None
            - number: int = 50
            - reverse: bool = True

        """
        return self.__base._request(
            "GET",
            route=f"/v2/conversations/{conversation_id}",
            params=params,
            data_type=PostsResponse,
        )

    def get_conversation_root_posts(self, post_ids: list[int]) -> PostsResponse:
        return self.__base._request(
            "GET",
            route=f"/v2/conversations/root_posts",
            params={"ids[]": post_ids},
            data_type=PostsResponse,
        )

    def get_following_call_timeline(self, **params) -> PostsResponse:
        """

        Parameters:
        ----------

            - from_timestamp: int = None
            - number: int = None
            - category_id: int = None
            - call_type: str = None
            - include_circle_call: bool = None
            - exclude_recent_gomimushi: bool = None

        """
        return self.__base._request(
            "GET",
            route=f"/v2/posts/call_followers_timeline",
            params=params,
            data_type=PostsResponse,
        )

    def get_following_timeline(self, **params) -> PostsResponse:
        """

        Parameters:
        ----------

            - from_str: str = None
            - only_root: bool = None
            - order_by: str = None
            - number: int = None
            - mxn: int = None
            - reduce_selfie: bool = None
            - custom_generation_range: bool = None

        """
        return self.__base._request(
            "GET",
            route=f"/v2/posts/following_timeline",
            params=params,
            data_type=PostsResponse,
        )

    def get_group_highlight_posts(self, group_id: int, **params) -> PostsResponse:
        """

        グループのまとめ投稿を取得します

        Parameters:
        ----------

            - group_id: int
            - from_post: int = None
            - number: int = None

        """
        return self.__base._request(
            "GET",
            route=f"/v1/groups/{group_id}/highlights",
            params=params,
            data_type=PostsResponse,
        )

    def get_group_timeline_by_keyword(
        self, group_id: int, keyword: str, **params
    ) -> PostsResponse:
        """

        Parameters:
        ----------

            - group_id: int
            - keyword: str
            - from_post_id: int = None
            - number: int = None
            - only_thread_posts: bool = False

        """
        params["keyword"] = keyword
        return self.__base._request(
            "GET",
            route=f"/v2/groups/{group_id}/posts/search",
            params=params,
            data_type=PostsResponse,
        )

    def get_group_timeline(self, group_id: int, **params) -> PostsResponse:
        """

        Parameters:
        ----------

            - group_id: int
            - from_post_id: int
            - reverse: bool
            - post_type: str
            - number: int
            - only_root: bool

        """
        params["group_id"] = group_id
        return self.__base._request(
            "GET",
            route=f"/v2/posts/group_timeline",
            params=params,
            data_type=PostsResponse,
        )

    def get_timeline_by_hashtag(self, hashtag: str, **params) -> PostsResponse:
        """

        Parameters:
        ----------

            - hashtag: str - (required)
            - from_post_id: int - (optional)
            - number: int - (optional)

        """
        return self.__base._request(
            "GET",
            route=f"/v2/posts/tags/{hashtag}",
            params=params,
            data_type=PostsResponse,
        )

    def get_my_posts(self, **params) -> PostsResponse:
        """

        Parameters:
        ---------------

            - from_post_id: int - (optional)
            - number: int - (optional)
            - include_group_post: bool - (optional)

        """
        return self.__base._request(
            "GET",
            route=f"/v2/posts/mine",
            params=params,
            data_type=PostsResponse,
        )

    def get_post(self, post_id: int) -> PostResponse:
        return self.__base._request(
            "GET", route=f"/v2/posts/{post_id}", data_type=PostResponse
        )

    def get_post_likers(self, post_id: int, **params) -> PostLikersResponse:
        """

        Parameters:
        ---------------

            - from_id: int - (optional)
            - number: int - (optional)

        """
        return self.__base._request(
            "GET",
            route=f"/v1/posts/{post_id}/likers",
            params=params,
            data_type=PostLikersResponse,
        )

    def get_post_reposts(self, post_id: int, **params: int) -> PostsResponse:
        """

        Parameters:
        ---------------

            - post_id: int - (required)
            - from_post_id: int - (optional)
            - number: int - (optional)

        """
        return self.__base._request(
            "GET",
            route=f"/v2/posts/{post_id}/reposts",
            params=params,
            data_type=PostsResponse,
        )

    def get_posts(self, post_ids: list[int]) -> PostsResponse:
        return self.__base._request(
            "GET",
            route=f"/v2/posts/multiple",
            params={"post_ids[]": post_ids},
            data_type=PostsResponse,
        )

    def get_recommended_post_tags(
        self, tag: str = None, save_recent_search: bool = False
    ) -> PostTagsResponse:
        return self.__base._request(
            "GET",
            route=f"/v1/posts/recommended_tag",
            payload={"tag": tag, "save_recent_search": save_recent_search},
            data_type=PostTagsResponse,
        )

    def get_recommended_posts(self, **params) -> PostsResponse:
        """

        Parameters:
        ---------------

            - experiment_num: int - (Required)
            - variant_num: int - (Required)
            - number: int - (Optional)

        """
        return self.__base._request(
            "GET",
            route=f"/v2/posts/recommended_timeline",
            params=params,
            data_type=PostsResponse,
        )

    def get_timeline_by_keyword(self, keyword: str = None, **params) -> PostsResponse:
        """

        Parameters:
        ---------------

            - keyword: str
            - from_post_id: int
            - number: int

        """
        params["keyword"] = keyword
        return self.__base._request(
            "GET",
            route=f"/v2/posts/search",
            params=params,
            data_type=PostsResponse,
        )

    def get_timeline(self, **params: int | str | bool) -> PostsResponse:
        # - from: str - (optional)
        """

        Parameters:
        ---------------

            - noreply_mode: bool - (optional)
            - from_post_id: int - (optional)
            - number: int - (optional)
            - order_by: str - (optional)
            - experiment_older_age_rules: bool - (optional)
            - shared_interest_categories: bool - (optional)
            - mxn: int - (optional)
            - en: int - (optional)
            - vn: int - (optional)
            - reduce_selfie: bool - (optional)
            - custom_generation_range: bool - (optional)

        """
        endpoint = f"/v2/posts/timeline"
        if "noreply_mode" in params and params["noreply_mode"] is True:
            endpoint = f"/v2/posts/noreply_timeline"
        return self.__base._request(
            "GET", route=endpoint, params=params, data_type=PostsResponse
        )

    def get_url_metadata(self, url: str) -> SharedUrl:
        return self.__base._request(
            "GET",
            route=f"/v2/posts/url_metadata",
            params={"url": url},
            data_type=SharedUrl,
        )

    def get_user_timeline(self, user_id: int, **params) -> PostsResponse:
        """

        Parameters:
        ---------------

            - from_post_id: int - (optional)
            - number: int - (optional)
            - post_type: str - (optional)

        """
        params["user_id"] = user_id
        return self.__base._request(
            "GET",
            route=f"/v2/posts/user_timeline",
            params=params,
            data_type=PostsResponse,
        )

    def like_posts(self, post_ids: list[int]) -> LikePostsResponse:
        return self.__base._request(
            "POST",
            route=f"/v2/posts/like",
            payload={"post_ids": post_ids},
            data_type=LikePostsResponse,
        )

    def remove_bookmark(self, user_id: int, post_id: int):
        return self.__base._request(
            "DELETE",
            route=f"/v1/users/{user_id}/bookmarks/{post_id}",
        )

    def remove_group_highlight_post(self, group_id: int, post_id: int):
        return self.__base._request(
            "DELETE",
            route=f"/v1/groups/{group_id}/highlights/{post_id}",
        )

    def remove_posts(self, post_ids: list[int]):
        return self.__base._request(
            "POST",
            route=f"/v2/posts/mass_destroy",
            payload={"posts_ids": post_ids},
        )

    def report_post(
        self,
        post_id: int,
        opponent_id: int,
        category_id: int,
        reason: str = None,
        screenshot_filename: str = None,
        screenshot_2_filename: str = None,
        screenshot_3_filename: str = None,
        screenshot_4_filename: str = None,
    ):
        return self.__base._request(
            "POST",
            route=f"/v3/posts/{post_id}/report",
            payload={
                "opponent_id": opponent_id,
                "category_id": category_id,
                "reason": reason,
                "screenshot_filename": screenshot_filename,
                "screenshot_2_filename": screenshot_2_filename,
                "screenshot_3_filename": screenshot_3_filename,
                "screenshot_4_filename": screenshot_4_filename,
            },
        )

    def unlike_post(self, post_id: int):
        return self.__base._request("POST", route=f"/v1/posts/{post_id}/unlike")

    def update_post(
        self,
        post_id: int,
        text: str = None,
        font_size: int = None,
        color: int = None,
        message_tags: list = [],
    ) -> Post:
        result = build_message_tags(text)
        if result is not None:
            text, message_tags = result
        return self.__base._request(
            "PUT",
            route=f"/v3/posts/{post_id}",
            payload={
                "text": text,
                "font_size": font_size,
                "color": color,
                "message_tags": message_tags,
                "api_key": Configs.API_KEY,
                "timestamp": int(datetime.now().timestamp()),
                "signed_info": self.__signed_info,
            },
        )

    def view_video(self, video_id: int):
        return self.__base._request(
            "POST",
            route=f"/v1/posts/videos/{video_id}/view",
        )

    def vote_survey(self, survey_id: int, choice_id: int) -> VoteSurveyResponse:
        return self.__base._request(
            "POST",
            route=f"/v2/surveys/{survey_id}/vote",
            payload={"choice_id": choice_id},
            data_type=VoteSurveyResponse,
        )

    @property
    def __signed_info(self) -> str:
        return md5(self.__base.device_uuid, int(datetime.now().timestamp()), False)



import unittest
import yaylib

from config import test_email, test_password


class BaseClientTests(unittest.TestCase):
    def setUp(self):
        self.base = yaylib.BaseClient()
        self.base.login(test_email, test_password)

    def test_make_request(self):
        pass



"""
MIT License

Copyright (c) 2023 qvco

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


class PostType(object):
    """投稿の種類"""

    text = "text"
    """ `text`: テキストのみ投稿タイプ"""
    media = "media"
    """ `media`: メディアを含める投稿タイプ"""
    image = "image"
    """ `image`: 画像を含める投稿タイプ"""
    video = "video"
    """ `video`: ビデオを含める投稿タイプ"""
    survey = "survey"
    """ `survey`: アンケートを含める投稿タイプ"""
    call = "call"
    """ `call`: 通話用の投稿タイプ"""
    shareable_url = "shareable_url"
    """ `shareable_url`: サークルやスレッド共有用の投稿タイプ"""


class CallType(object):
    """通話の種類"""

    voice = "voice"
    """ `voice`: 音声通話用の通話タイプ"""
    video = "vdo"
    """ `vdo`: ビデオ通話用の通話タイプ"""


class ImageType(object):
    """画像の種類"""

    post = "post"
    """ `post`: 投稿に画像をアップロードする際の画像タイプ"""
    chat_message = "chat_message"
    """ `chat_message`: 個人チャットに画像をアップロードする際の画像タイプ"""
    chat_background = "chat_background"
    """ `chat_background`: 個人チャットの背景用に画像をアップロードする際の画像タイプ"""
    report = "report"
    """ `report`: 通報用の画像をアップロードする際の画像タイプ"""
    user_avatar = "user_avatar"
    """ `user_avatar`: プロフィール画像をアップロードする際の画像タイプ"""
    user_cover = "user_cover"
    """ `user_cover`: プロフィールの背景画像をアップロードする際の画像タイプ"""
    group_cover = "group_cover"
    """ `group_cover`: グループの背景画像をアップロードする際の画像タイプ"""
    group_thread_icon = "group_thread_icon"
    """ `group_thread_icon`: グループ内のスレッド用アイコンをアップロードする際の画像タイプ"""
    group_icon = "group_icon"
    """ `group_icon`: グループのアイコンをアップロードする際の画像タイプ"""


class ShareableType(object):
    """共有の種類"""

    group = "group"
    """ `group`: サークル用の共有タイプ"""
    thread = "thread"
    """ `thread`: スレッド用の共有タイプ"""


class PolicyType(object):
    """利用規約の種類"""

    privacy_policy = "privacy_policy"
    """ `privacy_policy`: プライバシーポリシー"""
    terms_of_use = "terms_of_use"
    """ `privacy_policy`: 利用規約"""
